@@ -1,5 +1,5 @@
 """ 
-This modules contains functions to create and transform rotation matrices
+This modules contains functions to create and transform 3D rotation matrices
 and homogeneous tranformation matrices.
 
 Vector arguments are what numpy refers to as ``array_like`` and can be a list,
@@ -11,22 +11,23 @@ Versions:
     2. Josh Carrigg Hodson, Aditya Dua, Chee Ho Chan, 2017
     3. Peter Corke, 2020
     
-    TODO:
+TODO:
         
-        - trinterp
-        - trjac, trjac2
-        - tranimate, tranimate2
+    - trinterp
+    - trjac, trjac2
+    - tranimate, tranimate2
 """
 
 import sys
 import math
 import numpy as np
-import spatialmath.base.argcheck as argcheck
-import spatialmath.base.vectors as vec
-import spatialmath.base.transformsNd as trn
+from spatialmath.base import argcheck
+from spatialmath.base import vectors as vec
+from spatialmath.base import transformsNd as trn
 
-try:
-    print('Using SymPy')
+
+try: # pragma: no cover
+    # print('Using SymPy')
     import sympy as sym
     def issymbol(x):
         return isinstance(x, sym.Symbol)
@@ -67,6 +68,8 @@ def rotx(theta, unit="rad"):
     - ``rotx(THETA)`` is an SO(3) rotation matrix (3x3) representing a rotation
       of THETA radians about the x-axis
     - ``rotx(THETA, "deg")`` as above but THETA is in degrees
+    
+    :seealso: :func:`~trotx`
     """
 
     theta = argcheck.getunit(theta, unit)
@@ -94,6 +97,8 @@ def roty(theta, unit="rad"):
     - ``roty(THETA)`` is an SO(3) rotation matrix (3x3) representing a rotation
       of THETA radians about the y-axis
     - ``roty(THETA, "deg")`` as above but THETA is in degrees
+    
+    :seealso: :func:`~troty`
     """
 
     theta = argcheck.getunit(theta, unit)
@@ -121,6 +126,8 @@ def rotz(theta, unit="rad"):
     - ``rotz(THETA)`` is an SO(3) rotation matrix (3x3) representing a rotation
       of THETA radians about the z-axis
     - ``rotz(THETA, "deg")`` as above but THETA is in degrees
+    
+    :seealso: :func:`~yrotz`
     """
     theta = argcheck.getunit(theta, unit)
     ct = _cos(theta)
@@ -149,6 +156,8 @@ def trotx(theta, unit="rad", t=None):
       of THETA radians about the x-axis.
     - ``trotx(THETA, 'deg')`` as above but THETA is in degrees
     - ``trotx(THETA, 'rad', t=[x,y,z])`` as above with translation of [x,y,z]
+    
+    :seealso: :func:`~rotx`
     """
     T  = np.pad( rotx(theta, unit), (0,1), mode='constant' )
     if t is not None:
@@ -175,6 +184,8 @@ def troty(theta, unit="rad", t=None):
       of THETA radians about the y-axis.
     - ``troty(THETA, 'deg')`` as above but THETA is in degrees
     - ``troty(THETA, 'rad', t=[x,y,z])`` as above with translation of [x,y,z]
+    
+    :seealso: :func:`~roty`
     """
     T  = np.pad( roty(theta, unit), (0,1), mode='constant' )
     if t is not None:
@@ -201,6 +212,8 @@ def trotz(theta, unit="rad", t=None):
       of THETA radians about the z-axis.
     - ``trotz(THETA, 'deg')`` as above but THETA is in degrees
     - ``trotz(THETA, 'rad', t=[x,y,z])`` as above with translation of [x,y,z]
+    
+    :seealso: :func:`~rotz`
     """
     T  = np.pad( rotz(theta, unit), (0,1), mode='constant' )
     if t is not None:
@@ -234,6 +247,8 @@ def transl(x, y=None, z=None):
 
     - ``P = TRANSL(T)`` is the translational part of a homogeneous transform T as a
       3-element numpy array.
+    
+    :seealso: :func:`~spatialmath.base.transforms2d.transl2`
    """
 
     if np.isscalar(x):
@@ -266,7 +281,7 @@ def ishom(T, check=False, tol=10):
     - ``ISHOM(T, check=True)`` as above, but also checks orthogonality of the rotation sub-matrix and 
       validitity of the bottom row.
     
-    :seealso: isR, isrot, ishom2
+    :seealso: :func:`~spatialmath.base.transformsNd.isR`, :func:`~isrot`, :func:`~spatialmath.base.transforms2d.ishom2`
     """
     return T.shape == (4,4) and (not check or (trn.isR(T[:3,:3], tol=tol) and np.all(T[3,:] == np.array([0,0,0,1]))))
 
@@ -284,13 +299,13 @@ def isrot(R, check=False, tol=10):
     - ``ISROT(R)`` is True if the argument ``R`` is of dimension 3x3
     - ``ISROT(R, check=True)`` as above, but also checks orthogonality of the rotation matrix.
     
-    :seealso: isR, isrot2, ishom
+    :seealso: :func:`~spatialmath.base.transformsNd.isR`, :func:`~spatialmath.base.transforms2d.isrot2`,  :func:`~ishom`
     """
     return R.shape == (3,3) and (not check or trn.isR(R, tol=tol))
 
 
 # ---------------------------------------------------------------------------------------#
-def rpy2r(roll, pitch=None, yaw=None, unit='rad', order='zyx'):
+def rpy2r(roll, pitch=None, yaw=None, *, unit='rad', order='zyx'):
     """
     Create an SO(3) rotation matrix from roll-pitch-yaw angles
 
@@ -323,7 +338,9 @@ def rpy2r(roll, pitch=None, yaw=None, unit='rad', order='zyx'):
           
     - ``rpy2r(RPY)`` as above but the roll, pitch, yaw angles are taken
       from ``RPY`` which is a 3-vector (array_like) with values
-      (ROLL, PITCH, YAW). 
+      (ROLL, PITCH, YAW).
+      
+    :seealso: :func:`~eul2r`, :func:`~rpy2tr`, :func:`~tr2rpy`
     """
     
     if np.isscalar(roll):
@@ -384,6 +401,8 @@ def rpy2tr(roll, pitch=None, yaw=None, unit='rad', order='zyx'):
     Notes:
         
     - The translational part is zero.
+    
+    :seealso: :func:`~eul2tr`, :func:`~rpy2r`, :func:`~tr2rpy`
     """
 
     R = rpy2r(roll, pitch, yaw, order=order, unit=unit)
@@ -410,7 +429,9 @@ def eul2r(phi, theta=None, psi=None, unit='rad'):
       to rotations about the Z, Y, Z axes respectively.
     - ``R = eul2r(EUL)`` as above but the Euler angles are taken from
       ``EUL`` which is a 3-vector (array_like) with values
-      (PHI THETA PSI). 
+      (PHI THETA PSI).
+      
+    :seealso: :func:`~rpy2r`, :func:`~eul2tr`, :func:`~tr2eul`
     """
     
     if np.isscalar(phi):
@@ -449,6 +470,8 @@ def eul2tr(phi, theta=None, psi=None, unit='rad'):
     Notes:
         
     - The translational part is zero.
+
+    :seealso: :func:`~rpy2tr`, :func:`~eul2r`, :func:`~tr2eul`
     """
     
     R = eul2r(phi, theta, psi, unit=unit)
@@ -475,6 +498,8 @@ def angvec2r(theta, v, unit='rad'):
         
     - If ``THETA == 0`` then return identity matrix.
     - If ``THETA ~= 0`` then ``V`` must have a finite length.
+
+    :seealso: :func:`~angvec2tr`, :func:`~tr2angvec`
     """
     assert np.isscalar(theta) and argcheck.isvector(v, 3), "Arguments must be theta and vector"
     
@@ -512,6 +537,8 @@ def angvec2tr(theta, v, unit='rad'):
     - If ``THETA == 0`` then return identity matrix.
     - If ``THETA ~= 0`` then ``V`` must have a finite length.
     - The translational part is zero.
+
+    :seealso: :func:`~angvec2r`, :func:`~tr2angvec`
     """
     return trn.r2t(angvec2r(theta, v, unit=unit))
 
@@ -547,6 +574,8 @@ def oa2r(o, a=None):
     - O and A do not have to be unit-length, they are normalized
     - O and A do not have to be orthogonal, so long as they are not parallel
     - The vectors O and A are parallel to the Y- and Z-axes of the equivalent coordinate frame.
+
+    :seealso: :func:`~oa2tr`
     """
     o = argcheck.getvector(o, 3, out='array')
     a = argcheck.getvector(a, 3, out='array')
@@ -588,6 +617,8 @@ def oa2tr(o, a=None):
     - O and A do not have to be orthogonal, so long as they are not parallel
     - The translational part is zero.
     - The vectors O and A are parallel to the Y- and Z-axes of the equivalent coordinate frame.
+
+    :seealso: :func:`~oa2r`
     """
     return trn.r2t(oa2r(o, a))
 
@@ -615,8 +646,7 @@ def tr2angvec(T, unit='rad', check=False):
         
     - If the input is SE(3) the translation component is ignored.
     
-    :seealso: angvec2r, angvec2tr, tr2rpy, tr2eul
-    
+    :seealso: :func:`~angvec2r`, :func:`~angvec2tr`, :func:`~tr2rpy`, :func:`~tr2eul`
     """
 
     if argcheck.ismatrix(T, (4,4)):
@@ -669,7 +699,7 @@ def tr2eul(T, unit='rad', flip=False, check=False):
     - There is a singularity for the case where :math:`\theta=0` in which case :math:`\phi` is arbitrarily set to zero and :math:`\phi` is set to :math:`\phi+\psi`.
     - If the input is SE(3) the translation component is ignored.
     
-    :seealso: eul2r, eul2tr, tr2rpy, tr2angvec
+    :seealso: :func:`~eul2r`, :func:`~eul2tr`, :func:`~tr2rpy`, :func:`~tr2angvec`
     """
     
     if argcheck.ismatrix(T, (4,4)):
@@ -733,7 +763,7 @@ def tr2rpy(T, unit='rad', order='zyx', check=False):
     - There is a singularity for the case where P=:math:`\pi/2` in which case R is arbitrarily set to zero and Y is the sum (R+Y).
     - If the input is SE(3) the translation component is ignored.
     
-    :seealso: rpy2r, rpy2tr, tr2eul, tr2angvec
+    :seealso: :func:`~rpy2r`, :func:`~rpy2tr`, :func:`~tr2eul`, :func:`~tr2angvec`
     """
     
     if argcheck.ismatrix(T, (4,4)):
@@ -848,7 +878,7 @@ def trlog(T, check=True):
       vector (6x1) comprising [v w].
 
 
-    :seealso: trexp, vex, vexa
+    :seealso: :func:`~trexp`, :func:`~spatialmath.base.transformsNd.vex`, :func:`~spatialmath.base.transformsNd.vexa`
     """
     
     if ishom(T, check=check):
@@ -941,7 +971,7 @@ def trexp(S, theta=None):
       must represent a unit-twist, ie. the rotational component is a unit-norm skew-symmetric
       matrix.
      
-     :seealso: trlog, trexp2
+     :seealso: :func:`~trlog, :func:`~spatialmath.base.transforms2d.trexp2`
     """
    
     if argcheck.ismatrix(S, (4,4)) or argcheck.isvector(S, 6):
@@ -987,19 +1017,17 @@ def trexp(S, theta=None):
         raise ValueError(" First argument must be SO(3), 3-vector, SE(3) or 6-vector")
     
 
-
-
 def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', unit='deg'):
     """
-    Compact display of SO(2) or SE(2) matrices
+    Compact display of SO(3) or SE(3) matrices
     
     :param T: matrix to format
-    :type T: numpy.ndarray, shape=(2,2) or (3,3)
+    :type T: numpy.ndarray, shape=(3,3) or (4,4)
     :param label: text label to put at start of line
     :type label: str
     :param orient: 3-angle convention to use
     :type orient: str
-    :param file: file to write formatted string to
+    :param file: file to write formatted string to. [default, stdout]
     :type file: str
     :param fmt: conversion format for each number
     :type fmt: str
@@ -1009,14 +1037,9 @@ def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', uni
     :rtype: str
     
     The matrix is formatted and written to ``file`` or if ``file=None`` then the
-    string is returned. Orientation is expressed in one of several formats:
-    - 'rpy/zyx' roll-pitch-yaw angles in ZYX axis order [default]
-    - 'rpy/yxz' roll-pitch-yaw angles in YXZ axis order
-    - 'rpy/zyx' roll-pitch-yaw angles in ZYX axis order
-    - 'eul' Euler angles in ZYZ axis order
-    - 'angvec' angle and axis
-    
-    - ``trprint(R)`` displays the SO(3) rotation matrix in a compact 
+    string is returned. 
+
+   - ``trprint(R)`` displays the SO(3) rotation matrix in a compact 
       single-line format:
         
         [LABEL:] ORIENTATION UNIT
@@ -1024,7 +1047,16 @@ def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', uni
     - ``trprint(T)`` displays the SE(3) homogoneous transform in a compact 
       single-line format:
         
-        [LABEL:] [t=X, Y, Z;] ORIENTATION UNIT
+        [LABEL:] [t=X, Y, Z;] ORIENTATION UNIT    
+    
+    Orientation is expressed in one of several formats:
+        
+    - 'rpy/zyx' roll-pitch-yaw angles in ZYX axis order [default]
+    - 'rpy/yxz' roll-pitch-yaw angles in YXZ axis order
+    - 'rpy/zyx' roll-pitch-yaw angles in ZYX axis order
+    - 'eul' Euler angles in ZYZ axis order
+    - 'angvec' angle and axis
+    
 
     Example:
         
@@ -1041,8 +1073,9 @@ def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', uni
      - If the 'rpy' option is selected, then the particular angle sequence can be
        specified with the options 'xyz' or 'yxz' which are passed through to ``tr2rpy``.
        'zyx' is the default.
+     - Default formatting is for readable columns of data
       
-    :seealso: trprint2, tr2eul, tr2rpy, tr2angvec
+    :seealso: :func:`~spatialmath.base.transforms2d.trprint2`, :func:`~tr2eul`, :func:`~tr2rpy`, :func:`~tr2angvec`
     """
     
     s = ''
@@ -1062,11 +1095,11 @@ def trprint(T, orient='rpy/zyx', label=None, file=sys.stdout, fmt='{:8.2g}', uni
             seq = a[1]
         else:
             seq = None
-        angles = tr2rpy(t2r(T), order=seq, unit=unit)
+        angles = tr2rpy(T, order=seq, unit=unit)
         s += ' {} = {} {}'.format(orient, _vec2s(fmt, angles), unit)
         
     elif a[0].startswith('eul'):
-        angles = tr2eul(t2r(T), unit)
+        angles = tr2eul(T, unit)
         s += ' eul = {} {}'.format(_vec2s(fmt, angles), unit)
     
     elif a[0] == 'angvec':
@@ -1147,7 +1180,7 @@ try:
     
         # check input types
         if isrot(T, check=True):
-            T = r2t(T)
+            T = trn.r2t(T)
         else:
             assert ishom(T, check=True)
     
@@ -1216,17 +1249,19 @@ try:
             ax.text(x[0], x[1], x[2], "$%c_{%s}$" % (labels[0],frame), color=color, horizontalalignment='center', verticalalignment='center')
             ax.text(y[0], y[1], y[2], "$%c_{%s}$" % (labels[1], frame),  color=color, horizontalalignment='center', verticalalignment='center')
             ax.text(z[0], z[1], z[2], "$%c_{%s}$" % (labels[2], frame),  color=color, horizontalalignment='center', verticalalignment='center')
-except:
-    pass
+
+except:  # pragma: no cover
+    def trplot(*args, **kwargs):
+        print('** trplot: no plot produced -- matplotlib not installed')
     
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import pathlib
     import os.path
     
 
     
-    trplot( transl(1,2,3), frame='A', rviz=True, width=1, dims=[0, 10, 0, 10, 0, 10])
-    trplot( transl(3,1, 2), color='red', width=3, frame='B')
-    trplot( transl(4, 3, 1)@trotx(math.pi/3), color='green', frame='c', dims=[0,4,0,4,0,4])
+    # trplot( transl(1,2,3), frame='A', rviz=True, width=1, dims=[0, 10, 0, 10, 0, 10])
+    # trplot( transl(3,1, 2), color='red', width=3, frame='B')
+    # trplot( transl(4, 3, 1)@trotx(math.pi/3), color='green', frame='c', dims=[0,4,0,4,0,4])
     
-    #runfile(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_transforms.py") )
\ No newline at end of file
+    exec(open(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_transforms.py")).read() )
\ No newline at end of file
