@@ -16,11 +16,10 @@ import sys
 import math
 import numpy as np
 import numpy.matlib as matlib
-
-import spatialmath.base.argcheck as argcheck
 from spatialmath.base.vectors import *
-import spatialmath.base.transforms2d as t2d
-import spatialmath.base.transforms3d as t3d
+from spatialmath.base import transforms2d as t2d
+from spatialmath.base import transforms3d as t3d
+from spatialmath.base import argcheck
 
     
 _eps = np.finfo(np.float64).eps
@@ -448,15 +447,19 @@ def _rodrigues(w, theta):
     :param theta: rotation angle
     :type theta: float or None
     """
+    w = argcheck.getvector(w)
     if iszerovec(w):
-        # for a zero so(3) return unit matrix, theta not relevant
-        return np.eye(3)
+        # for a zero so(n) return unit matrix, theta not relevant
+        if len(w) == 1:
+            return np.eye(2)
+        else:
+            return np.eye(3)
     if theta is None:
         theta = norm(w)
         w = unitvec(w)
 
     skw = skew(w)
-    return np.eye(3) + math.sin(theta) * skw + (1.0 - math.cos(theta)) * skw @ skw
+    return np.eye(skw.shape[0]) + math.sin(theta) * skw + (1.0 - math.cos(theta)) * skw @ skw
 
 def h2e(v):
     """
@@ -507,11 +510,11 @@ def e2h(v):
         return np.vstack([v, np.ones((1,v.shape[1]))])
         
 
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import pathlib
     import os.path
     
-    runfile(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_transforms.py") )
+    exec(open(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_transforms.py")).read() )
     
     
 
